"use strict";
var byline = require("byline");
var util = require("util");
var LogcatHelper = (function () {
    function LogcatHelper($childProcess, $logcatPrinter, $logger) {
        this.$childProcess = $childProcess;
        this.$logcatPrinter = $logcatPrinter;
        this.$logger = $logger;
    }
    LogcatHelper.prototype.start = function (deviceIdentifier, adbPath) {
        var _this = this;
        this.$childProcess.exec(util.format("%s -s %s logcat -c", adbPath, deviceIdentifier)).wait();
        var adbLogcat = this.$childProcess.spawn(adbPath, ["-s", deviceIdentifier, "logcat"]);
        var lineStream = byline(adbLogcat.stdout);
        adbLogcat.stderr.on("data", function (data) {
            _this.$logger.trace("ADB logcat stderr: " + data.toString());
        });
        adbLogcat.on("close", function (code) {
            if (code !== 0) {
                _this.$logger.trace("ADB process exited with code " + code.toString());
            }
        });
        lineStream.on('data', function (line) {
            var lineText = line.toString();
            _this.$logcatPrinter.print(lineText);
        });
        return adbLogcat;
    };
    return LogcatHelper;
})();
exports.LogcatHelper = LogcatHelper;
$injector.register("logcatHelper", LogcatHelper);
