var options = require("./../common/options");
var iOSProxyServices = require("./../common/mobile/ios/ios-proxy-services");
var net = require("net");
var path = require("path");
var notification;
(function (notification) {
    function waitForDebug(bundleId) {
        return bundleId + ":NativeScript.Debug.WaitForDebugger";
    }
    notification.waitForDebug = waitForDebug;
    function attachRequest(bundleId) {
        return bundleId + ":NativeScript.Debug.AttachRequest";
    }
    notification.attachRequest = attachRequest;
    function appLaunching(bundleId) {
        return bundleId + ":NativeScript.Debug.AppLaunching";
    }
    notification.appLaunching = appLaunching;
    function readyForAttach(bundleId) {
        return bundleId + ":NativeScript.Debug.ReadyForAttach";
    }
    notification.readyForAttach = readyForAttach;
})(notification || (notification = {}));
var IOSDebugService = (function () {
    function IOSDebugService($platformService, $iOSEmulatorServices, $devicesServices, $platformsData, $projectData, $childProcess, $logger, $fs, $errors, $injector, $npm, $iosXcrunSimctl) {
        this.$platformService = $platformService;
        this.$iOSEmulatorServices = $iOSEmulatorServices;
        this.$devicesServices = $devicesServices;
        this.$platformsData = $platformsData;
        this.$projectData = $projectData;
        this.$childProcess = $childProcess;
        this.$logger = $logger;
        this.$fs = $fs;
        this.$errors = $errors;
        this.$injector = $injector;
        this.$npm = $npm;
        this.$iosXcrunSimctl = $iosXcrunSimctl;
    }
    Object.defineProperty(IOSDebugService.prototype, "platform", {
        get: function () {
            return "ios";
        },
        enumerable: true,
        configurable: true
    });
    IOSDebugService.prototype.debug = function () {
        if ((!options.debugBrk && !options.start) || (options.debugBrk && options.start)) {
            this.$errors.failWithoutHelp("Expected exactly one of the --debug-brk or --start options.");
        }
        if (options.emulator) {
            if (options.debugBrk) {
                return this.emulatorDebugBrk();
            }
            else if (options.start) {
                return this.emulatorStart();
            }
        }
        else {
            if (options.debugBrk) {
                return this.deviceDebugBrk();
            }
            else if (options.start) {
                return this.deviceStart();
            }
        }
        this.$errors.failWithoutHelp("Failed to select device or emulator to debug on.");
    };
    IOSDebugService.prototype.emulatorDebugBrk = function () {
        var _this = this;
        return (function () {
            var platformData = _this.$platformsData.getPlatformData(_this.platform);
            _this.$platformService.buildPlatform(_this.platform).wait();
            var emulatorPackage = _this.$platformService.getLatestApplicationPackageForEmulator(platformData).wait();
            _this.executeOpenDebuggerClient().wait();
            _this.$iOSEmulatorServices.startEmulator(emulatorPackage.packageName, { args: "--nativescript-debug-brk" }).wait();
        }).future()();
    };
    IOSDebugService.prototype.emulatorStart = function () {
        var _this = this;
        return (function () {
            var device = _this.getRunningEmulatorOrFail().wait();
            _this.executeOpenDebuggerClient().wait();
            var projectId = _this.$projectData.projectId;
            var attachRequestMessage = notification.attachRequest(projectId);
            device.postNotification(attachRequestMessage).wait();
        }).future()();
    };
    IOSDebugService.prototype.getRunningEmulatorOrFail = function () {
        var _this = this;
        return (function () {
            var device = _this.$iosXcrunSimctl.getRunningEmulator().wait();
            if (device) {
                _this.$logger.info("Using emulator: " + device.uuid);
            }
            else {
                _this.$errors.failWithoutHelp("This command expects the app to be running in an emulator.");
            }
            return device;
        }).future()();
    };
    IOSDebugService.prototype.getRunningEmulatorOrRunNew = function () {
        var _this = this;
        return (function () {
            var device = _this.$iosXcrunSimctl.getRunningEmulator().wait();
            if (device) {
                _this.$logger.info("Using emulator: " + device.uuid);
            }
            else {
                var devices = _this.$iosXcrunSimctl.getDevices().wait();
                var deviceToBoot = _.findLast(devices, function (d) { return d.available; });
                if (deviceToBoot) {
                    _this.$logger.info("Booting device: " + deviceToBoot.name + " (" + deviceToBoot.uuid + ")");
                    deviceToBoot.boot().wait();
                    return deviceToBoot;
                }
                else {
                    _this.$errors.failWithoutHelp("No available emulator to use.");
                }
            }
            return device;
        }).future()();
    };
    IOSDebugService.prototype.deviceDebugBrk = function () {
        var _this = this;
        return (function () {
            _this.$devicesServices.initialize({ platform: _this.platform, deviceId: options.device }).wait();
            _this.$devicesServices.execute(function (device) { return (function () {
                _this.$platformService.deployOnDevice(_this.platform).wait();
                var deviceDebugging = _this.$injector.resolve("iosDeviceDebugging", { bundleId: _this.$projectData.projectId, $iOSDevice: device });
                deviceDebugging.debugApplicationOnStart();
                _this.executeOpenDebuggerClient().wait();
                device.runApplication(_this.$projectData.projectId).wait();
            }).future()(); }).wait();
        }).future()();
    };
    IOSDebugService.prototype.deviceStart = function () {
        var _this = this;
        return (function () {
            _this.$devicesServices.initialize({ platform: _this.platform, deviceId: options.device }).wait();
            _this.$devicesServices.execute(function (device) { return (function () {
                _this.executeOpenDebuggerClient().wait();
                var deviceDebugging = _this.$injector.resolve("iosDeviceDebugging", { bundleId: _this.$projectData.projectId, $iOSDevice: device });
                deviceDebugging.debugRunningApplication();
            }).future()(); }).wait();
        }).future()();
    };
    IOSDebugService.prototype.executeOpenDebuggerClient = function () {
        var _this = this;
        if (options.client === false) {
            return (function () {
                _this.$logger.info("Supressing debugging client.");
            }).future()();
        }
        else {
            return this.openDebuggingClient();
        }
    };
    IOSDebugService.prototype.openDebuggingClient = function () {
        var _this = this;
        return (function () {
            var cmd = "open -a Safari " + _this.getSafariPath().wait();
            _this.$childProcess.exec(cmd).wait();
        }).future()();
    };
    IOSDebugService.prototype.getSafariPath = function () {
        var _this = this;
        return (function () {
            var tnsIosPackage = "";
            if (options.frameworkPath) {
                if (_this.$fs.getFsStats(options.frameworkPath).wait().isFile()) {
                    _this.$errors.failWithoutHelp("frameworkPath option must be path to directory which contains tns-ios framework");
                }
                tnsIosPackage = path.resolve(options.frameworkPath);
            }
            else {
                var platformData = _this.$platformsData.getPlatformData(_this.platform);
                tnsIosPackage = _this.$npm.install(platformData.frameworkPackageName).wait();
            }
            var safariPath = path.join(tnsIosPackage, "WebInspectorUI/Safari/Main.html");
            return safariPath;
        }).future()();
    };
    return IOSDebugService;
})();
$injector.register("iOSDebugService", IOSDebugService);
var IOSXcrunSimctlDevice = (function () {
    function IOSXcrunSimctlDevice(deviceName, deviceUUID, deviceBooted, deviceAvailable, $childProcess) {
        this.deviceName = deviceName;
        this.deviceUUID = deviceUUID;
        this.deviceBooted = deviceBooted;
        this.deviceAvailable = deviceAvailable;
        this.$childProcess = $childProcess;
    }
    Object.defineProperty(IOSXcrunSimctlDevice.prototype, "name", {
        get: function () {
            return this.deviceName;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IOSXcrunSimctlDevice.prototype, "uuid", {
        get: function () {
            return this.deviceUUID;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IOSXcrunSimctlDevice.prototype, "booted", {
        get: function () {
            return this.deviceBooted;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IOSXcrunSimctlDevice.prototype, "available", {
        get: function () {
            return this.deviceAvailable;
        },
        enumerable: true,
        configurable: true
    });
    IOSXcrunSimctlDevice.prototype.boot = function () {
        var _this = this;
        return (function () {
            try {
                _this.$childProcess.exec("xcrun instruments -w " + _this.uuid, { stdio: [null, process.stdout, process.stdout] }).wait();
                _this.deviceBooted = true;
            }
            catch (e) {
                console.log("Err: " + e);
            }
        }).future()();
    };
    IOSXcrunSimctlDevice.prototype.postNotification = function (notification) {
        return this.$childProcess.exec("xcrun simctl notify_post " + this.uuid + " " + notification);
    };
    IOSXcrunSimctlDevice.prototype.installApp = function (appPackage) {
        return this.$childProcess.exec("xcrun simctl install " + this.uuid + " " + appPackage);
    };
    IOSXcrunSimctlDevice.prototype.launchApp = function (appId, args) {
        var cmd = "xcrun simctl launch " + this.uuid + " " + appId;
        if (args) {
            cmd = cmd + " " + args;
        }
        return this.$childProcess.exec(cmd, { stdio: "inherit" });
    };
    return IOSXcrunSimctlDevice;
})();
$injector.register("iosXcrunSimctlDevice", IOSXcrunSimctlDevice);
var IOSXcrunSimctl = (function () {
    function IOSXcrunSimctl($injector, $childProcess) {
        this.$injector = $injector;
        this.$childProcess = $childProcess;
    }
    IOSXcrunSimctl.prototype.getRunningEmulator = function () {
        var _this = this;
        return (function () {
            return _.findLast(_this.getDevices().wait(), function (d) { return d.booted; });
        }).future()();
    };
    IOSXcrunSimctl.prototype.getDevices = function () {
        var _this = this;
        return (function () {
            var list = _this.$childProcess.exec("xcrun simctl list devices").wait();
            var devices = new Array();
            var matcher = /^\s*(.*?)\s*\((.*?)\)\s*(\(shutdown\)|\(booted\))\s*((\(unavailable\))?)\s*$/gim;
            var matches;
            while (matches = matcher.exec(list)) {
                devices.push(_this.$injector.resolve(IOSXcrunSimctlDevice, {
                    deviceName: matches[1],
                    deviceUUID: matches[2],
                    deviceBooted: matches[3].toLowerCase() === "(booted)",
                    deviceAvailable: !matches[4]
                }));
            }
            return devices;
        }).future()();
    };
    return IOSXcrunSimctl;
})();
$injector.register("iosXcrunSimctl", IOSXcrunSimctl);
var IOSDeviceDebugging = (function () {
    function IOSDeviceDebugging(bundleId, $iOSDevice, $logger, $injector) {
        this.bundleId = bundleId;
        this.$iOSDevice = $iOSDevice;
        this.$logger = $logger;
        this.$injector = $injector;
        this.$notificationProxyClient = this.$injector.resolve(iOSProxyServices.NotificationProxyClient, { device: this.$iOSDevice });
    }
    IOSDeviceDebugging.prototype.debugApplicationOnStart = function () {
        var _this = this;
        var appLaunchMessage = notification.appLaunching(this.bundleId);
        this.$notificationProxyClient.addObserver(appLaunchMessage, function () {
            _this.$logger.info("Got AppLaunching");
            _this.proxyDebuggingTraffic();
            var waitForDebuggerMessage = notification.waitForDebug(_this.bundleId);
            _this.$notificationProxyClient.postNotificationAndAttachForData(waitForDebuggerMessage);
        });
    };
    IOSDeviceDebugging.prototype.debugRunningApplication = function () {
        this.proxyDebuggingTraffic();
        var attachRequestMessage = notification.attachRequest(this.bundleId);
        this.$notificationProxyClient.postNotificationAndAttachForData(attachRequestMessage);
    };
    IOSDeviceDebugging.prototype.proxyDebuggingTraffic = function () {
        var _this = this;
        var identifier = this.$iOSDevice.getIdentifier();
        this.$logger.info("Device Identifier: " + identifier);
        var readyForAttachMessage = notification.readyForAttach(this.bundleId);
        this.$notificationProxyClient.addObserver(readyForAttachMessage, function () {
            _this.$logger.info("Got ReadyForAttach");
            var devicePort = 8080;
            var localPort = 8080;
            _this.printHowToTerminate();
            var localServer = net.createServer(function (localSocket) {
                _this.$logger.info("Front-end client connected to localhost " + localPort);
                var deviceSocket;
                var buff = new Array();
                localSocket.on('data', function (data) {
                    if (deviceSocket) {
                        deviceSocket.write(data);
                    }
                    else {
                        buff.push(data);
                    }
                });
                localSocket.on('end', function () {
                    _this.$logger.info('Localhost client disconnected!');
                    process.exit(0);
                });
                deviceSocket = _this.$iOSDevice.connectToPort(devicePort);
                _this.$logger.info("Connected localhost " + localPort + " to device " + devicePort);
                buff.forEach(function (data) {
                    deviceSocket.write(data);
                });
                deviceSocket.on('data', function (data) {
                    localSocket.write(data);
                });
                deviceSocket.on('end', function () {
                    _this.$logger.info("Device socket closed!");
                    process.exit(0);
                });
            });
            localServer.listen(localPort, function () {
                _this.$logger.info("Opened localhost " + localPort);
            });
        });
    };
    IOSDeviceDebugging.prototype.printHowToTerminate = function () {
        this.$logger.info("\nSetting up debugger proxy...\n\nPress Ctrl + C to terminate, or disconnect.\n");
    };
    return IOSDeviceDebugging;
})();
$injector.register("iosDeviceDebugging", IOSDeviceDebugging);
